unit MxGRAPHEMult;

interface

uses windows, sysUtils, graphics, classes, printers, math,
     Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs,
     jmmUtil, mxGraphe;

const
        IntSansSignif = -1;
        MaxVecteurMult = 256;
        CouleurAxe : Tcolor = clGray;
        NbreVoiesMax = 4;
        voieX = 0;
        extensionMX = '.MX';

type
        TNumeroVoie = 1..2;

        TCalculIncertitude = (ciNone,ciFreq,ciTemps,ciTension,ciDuty,ciPhase,ciTensionEff);
        indiceVecteurMult = 0..MaxVecteurMult;
        vecteurMult = array[indiceVecteurMult] of double;
        indiceVoieMult = voieX..NbreVoiesMax;

        TvoieRecord = record
                mini,maxi : double;
                Nom : string;
                Command,CommandMTX : ansiString;
                Unite,Signif : string;
                Active,Positive : boolean;
                CalculIncertitude : TCalculIncertitude;
                { mode multimtre }
                NumVoie : TnumeroVoie;
                duree : integer; // en ms
        end;

        TvoieHP = class
                mini,maxi : double;
                Nom : string;
                Command,CommandMTX : ansiString;
                Unite,Signif : string;
                Commentaire : string;
                Active,Positive : boolean;
                CalculIncertitude : TCalculIncertitude;
                NumVoie,NumVoieRef : TnumeroVoie;
                duree : integer; // ms
                constructor create;
                destructor Destroy;override;
end;

Tvoie = class(TvoieHP)
                a,b : double; { entier = rel*a+b }
                valeurCourante,oldValeur : double;
                incertitudeCourante : double;
                valeur,incertitude : vecteurMult; { mode multimtre }
                AxeLog : boolean;
                LabelVoie : Tlabel;
                GBvoie : TgroupBox;
                posCurseur : integer;
                constructor create;
                destructor Destroy; override;
                Function GrandeurToEcran(x : double) : integer;
                Procedure SetPhase;
                Procedure SetIncertitude;
                Function StrValeur(nombre : double) : string;
end;

Tgraphe = class
                 dimPoint : integer;
                 Canvas : Tcanvas;
                 Quadrillage : boolean;
                 ErreurLecture : boolean;
                 voie : array[indiceVoieMult] of Tvoie;
                 NbrePoints : integer;
                 LimiteCourbe : Trect;
                 PanelErreurLect : Tpanel;
                 MesureEnContinu : boolean;
                 Constructor Create;
                 Procedure DrawG(Arect : Trect);
                 Procedure EnregistrePointCourant;
                 Procedure AffichePointCourant(curseur : boolean);
                 Procedure TraceCroix(xr,yr,dxr,dyr : double;v : indiceVoieMult;Curseur : boolean);
                 procedure DrawAxis;
                 Procedure AfficheCurseur;
                 procedure SaveData(const NomFichier : string);
                 destructor Destroy; override;
                 procedure LineG(x1,y1,x2,y2 : integer);
                 procedure tri;
end;

const
    CouleurVoie : array[indiceVoieMult] of Tcolor = (clBlack,clRed,clGreen,clBlue,clPurple);
    VoieDefaut : array[indiceVoieMult] of TvoieRecord =
    ((mini:100;maxi:10000;Nom : 'f';command : 'FREQ';commandMTX : 'FREQ';
     unite : 'Hz';Signif : 'frquence';active : true;positive : true;
     calculIncertitude : ciFreq;Numvoie : 1;duree : 200),
     (mini:0;maxi:5;Nom : 'Ve';command : 'AC';commandMTX : 'AC';
     unite : 'V';signif : 'efficace';active : true;positive : true;
     calculIncertitude : ciTensionEff;Numvoie : 1;duree : 200),
    (mini:0;maxi:5;Nom : 'Vs';command : 'AC';commandMTX : 'AC';
     unite : 'V';signif : 'efficace';active : false;positive : true;
     calculIncertitude : ciTensionEff;Numvoie : 2;duree : 200),
     (mini:-180;maxi:+180;Nom : '';command : 'PHAS';commandMTX : 'PHAS';
     unite : '';signif : 'phase(1,2)';active : false;positive : false;
     calculIncertitude : ciPhase;Numvoie : 1;duree : 200),
     (mini:0;maxi:5;Nom : 'V';command : 'AMPL';commandMTX : 'AMPL';
     unite : 'V';signif : 'amplitude';active : false;positive : true;
     calculIncertitude : ciTension;Numvoie : 2;duree : 200));

var BaseDeTemps,DeltaT : double;
    FrequenceCourante : double;
    DeltaV : array[1..2] of double;

IMPLEMENTATION

constructor TvoieHP.Create;
begin
        inherited create;
        nom := '';
        Signif := '';
        Unite := '';
        Commentaire := '';
        Command := '';
        CommandMTX := '';
        Maxi := 1;
        Mini := 0;
        Active := false;
        Positive := true;
        CalculIncertitude := ciNone;
        NumVoie := 1;
        NumVoieRef := 1;
end;{--- TvoieHP.Create ---}

constructor Tvoie.Create;
begin
        inherited create;
        A := 1;
        B := 0;
        ValeurCourante := Nan;
        incertitudeCourante := 0;
        calculIncertitude := ciNone;
        NumVoie := 1;
        AxeLog := false;
end;{--- Tvoie.Create ---}

destructor Tvoie.Destroy;
begin
    inherited destroy
end;

destructor TvoieHP.Destroy;
begin
    inherited destroy
end;

Function Log(X:extended):extended;
const Loge = 0.434294481903251828;
Begin
   Log := Ln(X)*loge
End;

Function Dix(X:extended):extended;
const Ln10 = 2.302585092994045680;
Begin
   Dix := Exp(X*ln10)
End;

Function Tvoie.GrandeurToEcran(x : double) : integer;
begin
     try
     if axeLog then x := log(x);
     result := round(A*x+B)
     except
     result := intSansSignif;
     end
end;

Procedure Tgraphe.Tri;
{ Tri Shell Meyer-Baudoin p 456 }
{ origine vecteur en 0 }
var cle : double;
    i,j : integer;
    v : indiceVoieMult;
    SauvePoint : array[indiceVoieMult] of double;
const deuxPi = 360.0;
Begin
    i := 1;
    while (i<NbrePoints) do begin
        for v := voieX to NbreVoiesMax do
            sauvePoint[v] := voie[v].valeur[i];
        cle := voie[voieX].valeur[i];
        j := pred(i);
        while (j>=0) and (voie[voieX].valeur[j]>cle) do begin
             for v := voieX to NbreVoiesMax do
                 voie[v].valeur[succ(j)] := voie[v].valeur[j];
             dec(j);
        end;
        for v := voieX to NbreVoiesMax do
            voie[v].valeur[succ(j)] := sauvePoint[v];
        inc(i);
    end;
    for v := voieA to NbreVoiesMax do with voie[v] do
       if active and (unite='') then
          for i := 1 to pred(NbrePoints) do
              Valeur[i] := Valeur[i]-
                  round((Valeur[i]-Valeur[pred(i)])/DeuxPi)*DeuxPi
end;{tri}

procedure Tgraphe.EnregistrePointCourant;
var i : indiceVoieMult;
begin
    if ErreurLecture then begin
           PanelErreurLect.caption := 'Donnes incorrectes';
           PanelErreurLect.visible := true;
           exit;
    end;
{
    for i := voieX to NbreVoiesMax do with voie[i] do if active then
    if abs(valeurCourante-oldValeur)>(maxi-mini)/32 then begin
          PanelErreurLect.caption := 'Donnes instables (cf Avg)';
          PanelErreurLect.visible := true;
          exit;
    end;
}    
    PanelErreurLect.caption := '';
    for i := voieX to NbreVoiesMax do with voie[i] do if active then begin
        if isNan(valeurCourante) then exit;
        valeur[NbrePoints] := valeurCourante;
        incertitude[NbrePoints] := incertitudeCourante;
    end;
    inc(NbrePoints);
    AffichePointCourant(false);
end;

procedure Tgraphe.TraceCroix(xr,yr,dxr,dyr : double;v : indiceVoieMult;curseur : boolean);
var x,y,dx,dy,x1,x2,y1,y2 : integer;
begin
        x1 := voie[voieX].GrandeurToEcran(xr-dxr);
        x2 := voie[voieX].GrandeurToEcran(xr+dxr);
        if (x1<0) or (x2<0) then exit;
        dx := (x2-x1) div 2;
        if dx<dimPoint then dx := dimPoint;
        x := (x1+x2) div 2;
        y1 := voie[v].grandeurToEcran(yr-dyr);
        y2 := voie[v].grandeurToEcran(yr+dyr);
        if (y1<0) or (y2<0) then exit;
        dy := (y1-y2) div 2;
        if dy<dimPoint then dy := dimPoint;
        y := (y1+y2) div 2;
        Canvas.Pen.color := couleurVoie[v];
        Canvas.Brush.color := couleurVoie[v];
        if curseur
           then begin
              Canvas.MoveTo(x-3*dimPoint,y);
              Canvas.LineTo(x+3*dimPoint,y);
              Canvas.MoveTo(x,y-3*dimPoint);
              Canvas.LineTo(x,y+3*dimPoint);
           end
           else Canvas.RoundRect(x-dx,y-dy,x+dx,y+dy,1,1);
end;

procedure Tgraphe.AffichePointCourant(curseur : boolean);
var i : indiceVoieMult;
begin
    if ErreurLecture then exit;
    canvas.brush.style := bsSolid;
    if curseur
       then canvas.pen.width := 3
       else canvas.pen.width := 1;
    for i := voieA to NbreVoiesMax do if voie[i].active then
        TraceCroix(voie[voieX].valeurCourante,
                   voie[i].valeurCourante,
                   voie[voieX].incertitudeCourante,
                   voie[i].incertitudeCourante,i,curseur);
    canvas.pen.width := 1;
end;

procedure Tgraphe.DrawG(Arect : Trect);
var j : indiceVoieMult;
    i : integer;
begin
     LimiteCourbe := Arect;
     canvas.font.name := 'Calibri';
     canvas.font.size := 10;
     canvas.Pen.Mode := pmCopy;
     dec(LimiteCourbe.bottom,canvas.font.height+2);
     inc(LimiteCourbe.top,2);
     canvas.brush.color := clWindow;
     canvas.font.color := couleurAxe;
     canvas.FillRect(canvas.clipRect);
     canvas.brush.style := bsSolid;
     with voie[voieX],LimiteCourbe do begin
        dimPoint := succ((bottom-top) div 220);
        if axeLog
           then begin
                A := (right-left)/log(Maxi/Mini);
                B := left-log(Mini)*voie[voieX].A;
           end
           else begin
                A := (right-left)/(Maxi-Mini);
                B := left-Mini*voie[voieX].A;
           end;
     end;
     if quadrillage then drawAxis;
     canvas.Pen.style := psSolid;
     for j := voieA to NbreVoiesMax do with LimiteCourbe,voie[j] do if active then begin
         A := (top-bottom)/(Maxi-Mini);
         B := bottom-Mini*A;
     end;
     for i := 0 to pred(NbrePoints) do
         for j := voieA to NbreVoiesMax do if voie[j].active then 
           TraceCroix(voie[voieX].valeur[i],
              voie[j].valeur[i],
              voie[voieX].Incertitude[i],
              voie[j].Incertitude[i],j,false);
end;

destructor Tgraphe.destroy;
var j : indiceVoieMult;
begin
    for j := voieX to NbreVoiesMax do voie[j].free
end;

constructor Tgraphe.Create;
var j : indiceVoieMult;
begin
  inherited create;
  for j := voieX to NbreVoiesMax do begin
     voie[j] := Tvoie.create;
     with voie[j] do begin
              Nom := VoieDefaut[j].nom;
              Unite := VoieDefaut[j].unite;
              Signif := VoieDefaut[j].signif;
              Command := VoieDefaut[j].command;
              CommandMTX := VoieDefaut[j].commandMTX;
              Active := VoieDefaut[j].active;
              Positive := VoieDefaut[j].positive;
              Mini := VoieDefaut[j].mini;
              Maxi := VoieDefaut[j].maxi;
     end;
  end;
  voie[voieX].axeLog := true;
  quadrillage := true;
  erreurLecture := false;
  NbrePoints := 0;
end;

procedure Tgraphe.LineG(x1,y1,x2,y2 : integer);
begin
     canvas.moveTo(x1,y1);
     canvas.lineTo(x2,y2);
end;

procedure Tgraphe.DrawAxis;

procedure traceAxeLogx;
var i,j : integer;
    xs  : integer;
    intermediaire : boolean;
begin with canvas,limiteCourbe,voie[voieX] do begin
   Intermediaire := log(Maxi/Mini) <= 4;
   for i := round(log(Mini)) to round(log(Maxi)) do begin
	      xs := grandeurToEcran(dix(i));
        pen.style := psSolid;
        LineG(xs,top,xs,bottom);
        TextOut(xs,bottom+2,formatCourt(dix(i),Unite));
        pen.style := psDot;
        if intermediaire then for j := 2 to 9 do begin
	         xs := grandeurToEcran(dix(i+log(j)));
           LineG(xs,top,xs,bottom);
        end;{for j}
   end; {for i}
end end;{ TraceAxeLogx }

procedure traceAxeX;
var i : integer;
    xs  : integer;
    x : double;
begin with canvas,limiteCourbe,voie[voieX] do begin
   for i := 1 to 4 do begin
        x := i*Maxi/5;
       	xs := grandeurToEcran(x);
        LineG(xs,top,xs,bottom);
        TextOut(xs,bottom+2,formatCourt(x,unite));
   end; {for i}
end end;{ TraceAxeX }

var i,y : integer;
begin with canvas,LimiteCourbe do begin
     pen.color := CouleurAxe;
     pen.Style := psDot;
     pen.width := 1;
     for i := 0 to 10 do begin
          y := round(top+i*(bottom-top) div 10);
          LineG(left,y,right,y)
     end;
     if voie[voieX].axeLog then traceAxeLogX else traceAxeX;
     TextOut(right+2*font.height,bottom+2,voie[voieX].nom);
end end; // DrawAxis 

procedure Tgraphe.SaveData(const NomFichier : string);
var fichier : textFile;
    formatRW3 : boolean;

Procedure EcritPage;
var j : integer;
    i : indiceVoieMult;
begin if formatRW3
  then begin
     writeln(fichier,SymbReg,'1 PAGE COMMENT');
     writeln(fichier,'');
     writeln(fichier,SymbReg2,NbrePoints,' VALEUR VAR');
     for j := 0 to pred(NbrePoints) do begin
         write(fichier,FloatToStrF(voie[voieX].valeur[j],ffGeneral,7,1),' ');
         for i := succ(voieX) to NbreVoiesMax do if voie[i].active then
             write(fichier,FloatToStrF(voie[i].valeur[j],ffGeneral,5,1),' ');
        writeln(fichier);
     end
 end
 else begin
    for j := 0 to pred(NbrePoints) do begin
        for i := voieX to NbreVoiesMax do if voie[i].active then
             write(fichier,FloatToStrWin(voie[i].valeur[j]),crTab);
        writeln(fichier);
    end;
end
end; {ecritPage}

Procedure ecritEnTete;
var j : indiceVoieMult;
    N : integer;
begin if formatRW3
then begin
   N := 0;
   for j := voieX to NbreVoiesMax do if voie[j].active then inc(N);
   writeln(fichier,'EVARISTE REGRESSI WINDOWS 1.0');
   writeln(fichier,symbReg,N,' NOM VAR');
   for j := voieX to NbreVoiesMax do if voie[j].active then
       writeln(fichier,voie[j].nom);
   writeln(fichier,symbReg,N,' UNITE VAR');
   for j := voieX to NbreVoiesMax do if voie[j].active then
       writeln(fichier,voie[j].unite);
   writeln(fichier,symbReg,N,' SIGNIF VAR');
   for j := voieX to NbreVoiesMax do if voie[j].active then 
       writeln(fichier,voie[j].commentaire);
   writeln(fichier,symbReg,'1 TRIGO');
   writeln(fichier,ord(true));
   writeln(fichier,symbReg,'1 ACQUISITION');
   writeln(fichier,Application.ExeName)
end
else begin
   writeln(fichier,'HP54600');
   for j := voieX to NbreVoiesMax do if voie[j].active then
      write(fichier,voie[j].nom,crTab);
   writeln(fichier);
   for j := voieX to NbreVoiesMax do if voie[j].active then
      write(fichier,voie[j].unite,crTab);
   writeln(fichier);
   for j := voieX to NbreVoiesMax do if voie[j].active then
      write(fichier,voie[j].signif,crTab);
   writeln(fichier);
end
end; { ecritEnTete }

begin { saveData }
    FormatRW3 := AnsiUpperCase(ExtractFileExt(NomFichier))='.RW3';
    AssignFile(fichier,NomFichier);
    Rewrite(fichier);
    ecritEnTete;
    tri;
    ecritPage;
    CloseFile(fichier);
end;

procedure Tvoie.SetPhase;
begin
     if not isnan(valeurCourante) and
        (command='PHAS') then begin
      { oscillo renvoie entre -360 et +360 ramen entre -180 et +180 }
             if valeurCourante<-180 then valeurCourante := valeurCourante+360;
             if valeurCourante>180 then valeurCourante := valeurCourante-360;
         end;
end;

procedure Tvoie.SetIncertitude;
begin
      incertitudeCourante := 0;
      if not isnan(valeurCourante) then begin
      case calculIncertitude of
           ciNone : ;
           ciFreq : incertitudeCourante := valeurCourante*DeltaT*FrequenceCourante;
           ciTemps : incertitudeCourante := DeltaT;
           ciTension : incertitudeCourante := DeltaV[NumVoie];
           ciDuty : incertitudeCourante := DeltaT*FrequenceCourante;
           ciPhase :  incertitudeCourante := DeltaT*FrequenceCourante*360;
           ciTensionEff : incertitudeCourante := DeltaV[NumVoie]/4;
      end;
      end;
end;

Function Tvoie.StrValeur(nombre : double) : string;
var coeff : coefUnite;
    Negatif : boolean;
    tampon : string;
begin
     if isNan(nombre)
        then StrValeur := ''
        else begin
    negatif := nombre<0;
    nombre := abs(nombre);
    coeff := nulle;
    while ( nombre>=999 ) and (coeff<infini) do begin
          nombre := nombre/1000;
          inc(coeff)
    end; { nombre<1000 ou infini }
    while ( nombre<0.999 ) and (coeff>zero) do begin
          nombre := nombre*1000;
          Dec(coeff)
    end; { nombre>=1 ou zero }
    if coeff in [infini,zero]
      then tampon := '0 '
      else begin
         tampon := FloatToStrWin(nombre);
         if negatif then tampon := '-' + tampon;
         tampon := tampon + ' ';
         if coeff<>nulle then tampon := tampon+symboleUnite[coeff];
      end;
      StrValeur := tampon+unite;
   end;
end;

procedure Tgraphe.AfficheCurseur;
var x,y : integer;
    v : indiceVoieMult;
begin
    if ErreurLecture then exit;
    canvas.Pen.Mode := pmNotXor;
    canvas.pen.width := 3;
    canvas.brush.style := bsSolid;
    with voie[voieX] do begin
       posCurseur := GrandeurToEcran(valeurCourante);
       if posCurseur<0 then exit;
       x := posCurseur;
    end;
    for v := voieA to NbreVoiesMax do with voie[v] do if active then begin
        PosCurseur := grandeurToEcran(valeurCourante);
        y := posCurseur;
        if y>0 then begin
           Canvas.Pen.color := couleurVoie[v];
           Canvas.Brush.color := couleurVoie[v];
           Canvas.MoveTo(x-3*dimPoint,y);
           Canvas.LineTo(x+3*dimPoint,y);
           Canvas.MoveTo(x,y-3*dimPoint);
           Canvas.LineTo(x,y+3*dimPoint);
        end;   
    end;
    canvas.pen.width := 1;    
end; // AfficheCurseur

end.

