unit MxGraphe;

interface

uses windows, sysUtils, graphics, classes, printers,
     Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs, clipBrd, comctrls,
     math, jmmutil;

const
        maxVecteurOsc = 60000;

type
        Tmotif = (mCroix,mCroixDiag,mCercle,mCarre);
        indiceVecteurOsc = 0..MaxVecteurOsc;
        vecteurOsc = array[indiceVecteurOsc] of LongInt;
        vecteurOscR = array[indiceVecteurOsc] of double;

const
        IntSansSignif = -1;
        maxCol = 5;
        LongNombreAxe = 5;
        DimPointVGA : integer = 4; { dimension du point en pixel VGA }
        voieT = 0;
        voieA = 1;
        voieB = 2;
        NbreGraduations = 10;
        NbreGradMin = 5;
        voieMax = voieB;

type
       indiceVoieOsc = voieT..voieMax;

       TvoieOscillo = class
                Nom : string;
                Unite : string;
                calibre : double;
                Signif : string;
                Active : boolean;
                Gauge : TprogressBar;
                NomEdit,signifEdit : TlabeledEdit; 
                Valeur : vecteurOsc;
                origine : LongInt;
                offsetAna : double;
                increment : double;
                precision : integer;
                trLigne,trPoint : boolean;
                couleur : Tcolor;
                Motif : Tmotif;
                mini,maxi : double;
                xtitre,ytitre : integer;
                a,b : double; { entier = rel*a+b }
                deltaAxe,exposant,pointDebut : double;
              { deltaAxe = cart entre graduations sur les axes }
                NbreChiffres : byte;
                courbeBottom,courbeTop : integer;
                valeurReal : vecteurOscR;
                constructor create;
                procedure setDelta;
                procedure TrouveEchelle(fin : integer);
                function affecteEntier(r : double) : Integer;
                Function FormatMonde(x : double) : string;
                function chaineVolt(j : indiceVecteurOsc) : string;
                function valeurEtUnite(nombre : double) : string;
                procedure SetMonde(minX,maxX : double);
                function uniteAff(expos : double) : string;
  end;

  TgrapheOsc = class
         canvas : Tcanvas;
         Quadrillage : boolean;
         image : Timage;
         paintBox : TpaintBox;
         panel : Tpanel;
         dimPoint : integer;
         voie : array[indiceVoieOsc] of TvoieOscillo;
         LimiteCourbe,LimiteFenetre : Trect;
         LargCarac,HautCarac,Marge : integer;
         NbreOrdonnee : integer;
         penWidth : integer;
         NbreData : integer;
         Constructor Create;
         Procedure Draw;
         Procedure TraceCourbe(index : integer);
         Procedure WindowXY(xr,yr : double;m : indiceVoieOsc;var xi,yi : Integer);
         Procedure voieTY(xi,yi : integer;m : indiceVoieOsc;var xr,yr : double);
         procedure Segment(x1,y1,x2,y2 : integer);
         destructor Destroy; override;
         procedure SetHauteurFonte(H : integer;IsAxe : boolean);
         Procedure TraceMotif(xi,yi : integer;motif : Tmotif);
         procedure ChercheProjection;
         procedure imprime;
    end;

const
        FontBold : boolean = false;
        CouleurInit : array[indiceVoieOsc] of Tcolor =
 (clBlack,clBlue,clRed);
        MotifInit : array[indiceVoieOsc] of Tmotif =
 (mCroix,mCroix,mCroix);

var grapheOsc : TgrapheOsc;


IMPLEMENTATION

    procedure TgrapheOsc.Segment(x1,y1,x2,y2 : integer);
    begin
       canvas.moveTo(x1,y1);
       canvas.lineTo(x2,y2);
    end;

   procedure MaxiMini(vect : vecteurOscR;var max,min : double;Debut,Fin : integer);
   var j : integer;
   begin
      Max := vect[debut];min := max;
      for j := succ(debut) to Fin do
          if vect[j]>Max
             then Max := vect[j]
             else if vect[j]<Min
                  then Min := vect[j];
   end; // maxiMini

constructor TvoieOscillo.Create;
begin
    inherited create;
    signif := 'tension';
    active := true;
    unite := 'V';
    precision := 5;
    offsetAna := 0;
end; // TvoieOscillo.Create

Procedure TvoieOscillo.TrouveEchelle(Fin : integer);
var j : integer;
begin
    for j := 0 to Fin do
        valeurReal[j] := (valeur[j]-origine)*Increment-offsetAna;
    Mini := -origine*increment-offsetAna;
    Maxi := Mini+calibre;
end; // TvoieOscillo.Trouve 

function TvoieOscillo.affecteEntier(r : double) : Integer;
begin
      if isNan(r)
         then result := IntSansSignif
         else try
            result := round(A*r+B);
            except
               result := IntSansSignif
            end
end;

const Nbre125 : array[1..3] of integer = (1,2,5);

Procedure TgrapheOsc.voieTY(xi,yi : integer;m : indiceVoieOsc;var xr,yr : double);
begin
     xr := (xi-voie[voieT].B)/voie[voieT].A;
     yr := (yi-voie[m].B)/voie[m].A
end;

Procedure TgrapheOsc.WindowXY(xr,yr : double;m : indiceVoieOsc;var xi,yi : Integer);
begin
      yi := voie[m].affecteEntier(yr);
      xi := voie[voieT].affecteEntier(xr)
end;

Procedure TgrapheOsc.ChercheProjection;

  procedure chercheProjectionY(m : indiceVoieOsc);
  begin with voie[m] do begin
       if abs(Mini)<(Maxi-Mini)*1e-2 then Mini := 0;
       A := (CourbeTop-CourbeBottom)/(Maxi-Mini);
       B := CourbeBottom-Mini*A;
       setDelta;
  end end; { chercheProjectionY }

  procedure chercheProjectionX;
  begin with voie[voieT] do begin
        if abs(Mini)<(Maxi-Mini)*1e-2 then Mini := 0;
        A := (limiteCourbe.right-limiteCourbe.left)/(Maxi-Mini);
        B := limiteCourbe.left-Mini*A;
        setDelta;
  end end;

var m : indiceVoieOsc;
    decalage : integer;
begin { ChercheMonde }
     for m := voieT to voieMax do if voie[m].active then
         voie[m].trouveEchelle(NbreData);
     limiteCourbe := limiteFenetre;
     inc(limiteCourbe.left,Marge);
     dec(limiteCourbe.bottom,Marge);
        inc(limiteCourbe.top,hautCarac+Marge);
     dec(limiteCourbe.right,Marge);
     decalage := succ(voie[voieA].NbreChiffres)*largCarac;
     inc(limiteCourbe.left,decalage);
     if voie[voieB].active then
              dec(limiteCourbe.right,succ(voie[voieB].NbreChiffres)*largCarac);
     dec(limiteCourbe.bottom,2*HautCarac);
     for m := voieA to voieMax do with voie[m] do begin
         CourbeTop := LimiteCourbe.top;
         CourbeBottom := LimiteCourbe.bottom;
     end;
     chercheProjectionX;
     for m := voieA to voieMax do
         if voie[m].active then chercheProjectionY(m);
end; // ChercheProjection 

procedure TvoieOscillo.SetMonde(minX,maxX : double);
begin
       Maxi := MaxX;
       Mini := MinX;
end; 

procedure TgrapheOsc.TraceCourbe(index : integer);
var couleurLoc : Tcolor;

Procedure TracePoint;
var xi,yi : Integer;
    i : integer;
begin
    for i := 0 to pred(NbreData) do begin
    WindowXY(voie[voieT].valeurReal[i],voie[index].valeurReal[i],index,xi,yi);
    if (yi>0) and (xi>0) then case voie[index].Motif of
                   mCroix : begin
                       Segment(xi-dimPoint,yi,xi+dimPoint,yi);
                       Segment(xi,yi-dimPoint,xi,yi+dimPoint);
                   end;
                   mCroixDiag : begin
                       segment(xi-dimPoint,yi-dimPoint,xi+dimPoint,yi+dimPoint);
                       segment(xi+dimPoint,yi-dimPoint,xi-dimPoint,yi+dimPoint);
                   end;
                   mCercle : canvas.ellipse(xi-dimPoint,yi-dimPoint,xi+dimPoint+1,yi+dimPoint+1);
                   mCarre : canvas.rectangle(xi-dimPoint,yi-dimPoint,xi+dimPoint,yi+dimPoint);
          end;{case}
end end; // tracePoint

Procedure traceLigne;
var xi,yi : Integer;
    i : integer;
begin
     WindowXY(voie[voieT].valeurReal[0],voie[index].valeurReal[0],index,xi,yi);
     canvas.LineTo(xi,yi);
     for i := 1 to pred(NbreData) do begin
        WindowXY(voie[voieT].valeurReal[i],voie[index].valeurReal[i],index,xi,yi);
        if (xi>0) and (yi>0) then canvas.MoveTo(xi,yi);
     end;{for i}
end; // traceLigne

// TgrapheOsc.TraceCourbe(index)
begin
      couleurLoc := ColorToRGB(voie[index].couleur);
      if voie[index].trPoint then begin
            canvas.pen.style := psSolid;
            canvas.pen.width := penWidth;
            canvas.pen.color := couleurLoc;
            tracePoint;
      end;
      TraceLigne;
end; // TraceCourbe 

procedure TgrapheOsc.SetHauteurFonte(H : integer;IsAxe : boolean);
begin with canvas.Font do begin
     if IsAxe then
        if H<=13
           then H := 13 { MS sans serif 8 points = 13 pixels }
           else if H<=18
               then H := 16 { MS sans serif 10 points = 16 pixels }
               else if H<=24
                    then H := 20; { MS sans serif 12 points = 20 pixels }
     Height := H;
end end;

procedure TgrapheOsc.Draw;
var c : integer;
begin
     with limiteFenetre do begin
        dimPoint := (bottom-top)*dimPointVGA div 480;
        if dimPoint=0 then dimPoint := 1;
        penWidth := 2*(bottom-top) div 480;
        if penWidth<2 then penWidth := 2;
        setHauteurFonte((bottom-top) div 30,true);
     end;
     if FontBold
        then canvas.Font.style := [fsBold]
        else canvas.Font.style := [];
     canvas.pen.color := clBlack;
     LargCarac := canvas.TextWidth('A');
     HautCarac := canvas.TextHeight('A');
     Marge := largCarac div 2;
     chercheProjection;
     for c := voieA to voieMax do if voie[c].active then traceCourbe(c);
end; // draw

Procedure DebutImpressionGr(Orientation : TprinterOrientation);
begin
     printer.Orientation := Orientation;
     printer.title := 'Regressi';
     Printer.Canvas.Font.Name := 'Calibri';
     Printer.Canvas.Font.Size := 12;
     Printer.Canvas.Font.Color := clBlack;
     printer.beginDoc;
end;

procedure TgrapheOsc.imprime;
const  MargeImprimante = 21;
       MargeImprimantePaysage = 30; // soit 1 cm
var PageW,pageH,margeI : integer;
    rgnGraphe: Hrgn;
begin
     debutImpressionGr(poPortrait);
     canvas := printer.canvas;
     limiteFenetre := Printer.Canvas.ClipRect;
     PageW := GetDeviceCaps (Printer.Handle,PHYSICALWIDTH);
     PageH := GetDeviceCaps (Printer.Handle,PHYSICALHEIGHT);
     if printer.orientation = poPortrait
        then begin
           margeI := PageW div margeImprimante;
           with limiteFenetre do begin
             right := right - margeI;
             left := left + margeI;
             top := margeI div 2;
             bottom := top + (pageH div 2);
           end;
        end
        else begin
           margeI := PageW div margeImprimantePaysage;
           with limiteFenetre do begin
               top := top + (margeI div 2);
               bottom := bottom - margeI;
               right := right - margeI;
               left := left + margeI;
           end;
        end;
  with limiteFenetre do
       rgnGraphe := createRectRgn(left, top, right, bottom);
  selectClipRgn(Printer.Canvas.handle, rgnGraphe);
  draw;
  selectClipRgn(Printer.Canvas.handle, 0);{RZ}
  DeleteObject(rgnGraphe);
  printer.endDoc;
end;

destructor TgrapheOsc.destroy;
var m : indiceVoieOsc;
begin
     for m := voieT to voieMax do
         voie[m].free;
     inherited destroy;
end;

constructor TgrapheOsc.Create;
var m : indiceVoieOsc;
    j : integer;
begin
  inherited create;
  for m := voieT to voieMax do begin
       voie[m] := TvoieOscillo.create;
       with voie[m] do begin
         Couleur := couleurInit[m];
         motif := motifInit[m];
       end;
  end;
  for j := 0 to maxVecteurOsc do
      voie[voieT].valeur[j] := j;
  voie[voieA].nom := 'VA';
  voie[voieB].nom := 'VB';
  voie[voieT].nom := 't';
  voie[voieT].signif := 'temps';
  voie[voieT].unite := 's';
end;

procedure TvoieOscillo.SetDelta;
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }

const
    indexMax = 5;
    coeff : array[0..indexMax] of double = (0.1,0.2,0.5,1,2,5);
var  AbsMax,AbsMin,largeur : double;
     Exponent,N,index : integer;
begin
                AbsMax := abs(maxi);AbsMin := abs(mini);
          if AbsMax<AbsMin then AbsMax := AbsMin;
              Exponent := floor(Log10(Maxi-Mini));
              largeur := (maxi-mini)*power(10,-exponent);
              index := 0;
       while ( (largeur/coeff[index])>=NbreGraduations ) and
                    (index<indexMax) do
                 inc(index);
      DeltaAxe := coeff[index]*power(10,exponent);
   NbreChiffres := ceil(Log10(AbsMax))-floor(Log10(DeltaAxe));
   PointDebut := ceil(mini/deltaAxe)*deltaAxe;
   Index := Exponent div 3;
   if ( (Exponent mod 3) < 0) then dec(index);
  { la division euclidienne des informaticiens <> div des maths ! }
   Exposant := power(10,3*Index);
         N := ceil(Log10(AbsMax/Exposant));
         if N>NbreChiffres then NbreChiffres := N;
         if NbreChiffres<3 then NbreChiffres := 3;
         if NbreChiffres>longNombreAxe then
            NbreChiffres := longNombreAxe;
end; { setDelta }

Function TvoieOscillo.FormatMonde(x : double) : string;
begin
      result := FloatToStrF(x,ffGeneral,NbreChiffres,0)
end;

Procedure TgrapheOsc.TraceMotif(xi,yi : integer;motif : Tmotif);
begin
     case Motif of
          mCroix : begin
              segment(xi-dimPoint,yi,xi+dimPoint,yi);
              segment(xi,yi-dimPoint,xi,yi+dimPoint);
          end;
          mCroixDiag : begin
              segment(xi-dimPoint,yi-dimPoint,xi+dimPoint,yi+dimPoint);
              segment(xi+dimPoint,yi-dimPoint,xi-dimPoint,yi+dimPoint);
          end;
          mCercle : canvas.ellipse(xi-dimPoint,yi-dimPoint,xi+dimPoint+1,yi+dimPoint+1);
          mCarre : canvas.rectangle(xi-dimPoint,yi-dimPoint,xi+dimPoint,yi+dimPoint);
     end;
end;

function AjoutePuissDix(nom: string; puiss: shortint): string;
var
   dest: string;
begin
   if puiss = 0 then
      ajoutePuissDix := nom
   else begin
      dest := '10' + puissToStr(puiss) + ' ';
      if length(dest) + Length(nom) > 16 then
         ajoutePuissDix := dest + 'S.I.'
      else
         ajoutePuissDix := dest + nom;
   end;
end;

function TvoieOscillo.chaineVolt(j : indiceVecteurOsc) : string;
begin
     result := FloatToStrF((valeur[j]-origine)*Increment-offsetAna,ffGeneral,precision,1)
end;

Function TvoieOscillo.ValeurEtUnite(Nombre : double) : string;
var unite : coefUnite;
    tampon : string;
begin
    result := '';
    if isNan(nombre) then exit;
    unite := nulle;
    while ( Nombre>=999 ) and (unite<infini) do begin
          Nombre := Nombre/1000;
          inc(unite)
    end; { nombre<1000 ou infini }
    while ( Nombre<0.999 ) and (unite>zero) do begin
          Nombre := Nombre*1000;
          Dec(unite)
    end; { nombre>=1 ou zero }
    tampon := FloatToStrF(Nombre,ffGeneral,3,0);
    tampon := tampon+' '+uniteAff(puissanceUnite[unite]);
    result := tampon;
end;

Function TvoieOscillo.UniteAff(expos : double) : string;

Function AjoutePrefixe(nom : string;P : shortInt) : string;
const
    lettreUnite : TSysCharSet = ['A','C','F','H','J','K','L','N','P','S','T','V','W',
                              'b','c','d','g','l','m','r','s',OmegaMaj];
var prefixe : char;
begin
     case P of
          -18 : prefixe := 'a';
          -15 : prefixe := 'f';
          -12 : prefixe := 'p';
          -9 : prefixe := 'n';
          -6 : prefixe := '';
          -3 : prefixe := 'm';
          -2 : prefixe := 'c';
          -1 : prefixe := 'd';
          0  : begin
             ajoutePrefixe := nom;
             exit;
          end;
          +2 : prefixe := 'h';
          +3 : prefixe := 'k';
          +6 : prefixe := 'M';
          +9 : prefixe := 'G';
          +12 : prefixe := 'T';
          +15 : prefixe := 'P';
          +18 : prefixe := 'E';
          else begin
            ajoutePrefixe := AjoutePuissDix(nom,P);
            exit;
          end;
      end;
      ajoutePrefixe := prefixe+nom;
end;

begin
    result := AjoutePrefixe(Unite,round(log10(expos)))
end;

end.

